import os

class Experiment: 
    def __init__(self, alias: str, a: int, b: int, step_size: int, experiment_type: str): 
        from .load_config import load_config
        
        self.alias = str(alias)
        self.a = int(a)
        self.b = int(b)
        self.step_size = int(step_size)
        self.experiment_type = str(experiment_type)

        # Load configuration
        config = load_config()
        root_path = config.get("data_files_path", "data_files")  # Default to "data_files" if not set in config

        # Define name and path for saving results
        self.name = f"{self.alias}-{self.a},{self.b}-{self.step_size}-{self.experiment_type}.txt"
        self.path = os.path.join(root_path, self.experiment_type, self.alias, self.name)

        # Initialize seed_data and density_data based on experiment type
        if experiment_type in ["num", "sym"]:
            self.seed_data = []
        self.density_data = {"efficacy": [], "radius": []}

        # Ensure directories exist
        os.makedirs(os.path.dirname(self.path), exist_ok=True)
    
    def run_experiment(self, max_radius: int) -> None:
        from ._rp_num import _rp_num
        from ._rp_ff import _rp_ff
        #from .__rp_sym__ import __rp_sym__

        match self.experiment_type: 
            case "num":
                self.density_data, self.seed_data = _rp_num(self.a, self.b, self.step_size, max_radius, self)
            case "ff":
                self.density_data = _rp_ff(self.a, self.b, self.step_size, max_radius, self)
            case "sym":
                pass 
                #self.density_data, self.seed_data = __rp_sym__(self.a, self.b, self.step_size, max_radius, self)
            case _:
                raise ValueError("invalid experiment type")
        print("Experiment done and added to instance")

    def write_to_file(self) -> None:
        with open(self.path, "w") as file:
            file.write(f"alias:{self.alias}\n")
            file.write(f"a:{self.a}\n")
            file.write(f"b:{self.b}\n")
            file.write(f"step_size:{self.step_size}\n")
            file.write(f"experiment_type:{self.experiment_type}\n")
            
            file.write("--- Seed Data ---\n")
            for seed in self.seed_data:
                file.write(f"{seed}\n")
            
            file.write("--- Density Data ---\n")
            for efficacy, radius in zip(self.density_data["efficacy"], self.density_data["radius"]):
                file.write(f"{efficacy},{radius}\n")
        
        print("Data written to file")

    def get_meta_data(self) -> dict[str,int|str]:
        return {'alias': self.alias,
                'a': self.a,
                'b': self.b,
                'experimentType': self.experiment_type,
                'step_size': self.step_size,
                'max_radius': self.get_max_radius()}

    def get_max_radius(self) -> int: 
        try: 
            return int(self.density_data["radius"][-1])
        except IndexError:
            print("seed_data is empty")
            return None
    
    def get_density(self) -> list[float]: 
        return [float(e) / (float(r) ** 2) for e, r in zip(self.density_data['efficacy'], self.density_data['radius'])]

    def get_seed_data(self) -> list[dict[str,float|int]]: 
        return self.seed_data
        
    def get_radius(self) -> list[int]:
        return self.density_data["radius"]

    def get_efficacy(self) -> list[int]:
        return self.density_data["efficacy"]
        
    @staticmethod
    def read_from_file(alias: str, a: int, b: int, step_size: int, experiment_type: str):
        import rotational_placement.load_config as load_config
        
        # Load configuration for the path
        config = load_config()
        root_path = config.get("data_files_path", "data_files")  # Default to "data_files" if not set in config

        # Create the file name and path
        name = f'{alias}-{a},{b}-{step_size}-{experiment_type}.txt'
        path = os.path.join(root_path, experiment_type, alias, name)

        seed_data = []
        density_data = {'efficacy': [], 'radius': []}
        section = None

        # Read the data from the file
        with open(path, 'r') as file:
            for line in file:
                line = line.strip()
                if line == '--- Seed Data ---':
                    section = 'seed_data'
                elif line == '--- Density Data ---':
                    section = 'density_data'
                elif section == 'seed_data':
                    seed_data.append(line)
                elif section == 'density_data':
                    if line:  # Avoid empty lines
                        efficacy, radius = map(float, line.split(','))
                        density_data['efficacy'].append(efficacy)
                        density_data['radius'].append(radius)

        # Create and return an Experiment instance
        experiment = Experiment(alias, a, b, step_size, experiment_type)
        experiment.seed_data = seed_data
        experiment.density_data = density_data
        return experiment



from .experiment_class import Experiment
def plot_flower(experiment: Experiment, max_radius=0):
    """
    Description
    -----------
    plots a circle of either specified radius or the max radius of given experiment and saves that as a flower plot

    PARAMETERS
    ----------
    experiment: Experiment
        instance with seed_data that is to be visualized
    max_radius: int
        default is 0, if specified max_radius is desired input max_radius

    RETURNS
    -------
    The function saves the plot at specified location (default is ./plots/flower_plots/)
    """

    if max_radius == 0: 
        max_radius = experiment.get_max_radius()
    
    from .load_config import load_config
    root_path = load_config().get("plot_save_path","plots")

    name = f"flower-{experiment.get_meta_data['alias']}-{max_radius}.png"
    path = f'{root_path}/flower_plots/{name}'

    import matplotlib.pyplot as plt
    _,ax = plt.subplots()

    ax.set_ylim(-max_radius * 1.1, max_radius * 1.1)
    ax.set_xlim(-max_radius * 1.1, max_radius * 1.1)

    ax.set_aspect('equal',adjustable='box')
    ax.set_axis_off()

    ax.add_patch(plt.Circle((0,0),max_radius,fill=False,color='k'))

    print('...creating plot...')

    for seed in experiment.getSeedData(): 
        if seed["distance"] > max_radius:
            break
        ax.add_patch(plt.Circle(seed,1,fill=True,color='k'))


    plt.savefig(path)    
    print(f"flower saved at {path}")

    


import os

def make_data_dir():
    """
    Description
    -----------
    creates a data_files directory and sub-directories with apt names in current repo
    """
    # Define the main folders to create
    subfolders = ["num", "ff", "sym"]
    
    # Create the base directory
    os.makedirs("data_files", exist_ok=True)
    
    # Loop over each folder and create it inside the base directory
    for folder in subfolders:
        folder_path = os.path.join("data_files", folder)
        os.makedirs(folder_path, exist_ok=True)


import os

def make_plot_dir():
    """
    Description
    -----------
    creates a plots directory and sub-directories with apt names in current repo
    """
    # Define the subdirectories to create under 'plots'
    subfolders = ["density_plots", "flower_plots", "heatmaps"]
    
    # Create the base directory
    os.makedirs("plots", exist_ok=True)
    
    # Loop over each subfolder and create it inside the base directory
    for folder in subfolders:
        folder_path = os.path.join("plots", folder)
        os.makedirs(folder_path, exist_ok=True)


from .experiment_class import Experiment

def plot_density(aliass:list[str], *experiments: Experiment):
    """
    DESCRIPTION
    -----------
    Plot densities for multiple experiments, only including the common radius range.
    
    PARAMETERS
    ----------
    aliass: list[str]
        list of labels for data sets
    experiments: Experiment instances
        Instances of the Experiment class.
        
    RETURNS 
    -------
    The function saves the plot at specified location (default is ./plots/density_plots/)
    """

    import matplotlib.pyplot as plt
    import numpy as np
    import os

    # Find the maximum common radius across all experiments
    max_common_radius = min([max(exp.getRadius()) for exp in experiments])

    # Construct the name for saving the plot
    name = 'density-' + '-'.join([exp.getMetaData()['alias'] for exp in experiments]) + '.svg'
    path = f'plots/densityPlots/{name}'

    # Ensure the directory exists
    os.makedirs(os.path.dirname(path), exist_ok=True)

    # Create the plot
    f, ax = plt.subplots(1)
    
    # Loop through each experiment instance
    for i,exp in enumerate(experiments):
        radius = np.array(exp.getRadius())
        efficacy = np.array(exp.getEfficacy())
        
        # Filter data to only include points within the common radius
        valid_indices = radius <= max_common_radius
        truncated_radius = radius[valid_indices]
        truncated_efficacy = efficacy[valid_indices]
        
        # Calculate density for the truncated data
        density = truncated_efficacy / (truncated_radius ** 2)
        
        # Plot with the alias as the label
        ax.plot(truncated_radius, density, label=aliass[i])
    
    # Set axis limits
    ax.set_ylim(ymin=0, ymax=1.05)
    ax.set_xlim(xmin=0, xmax=max_common_radius)
    
    # Adjust plot appearance
    for spine in ["bottom", "left", "top", "right"]:
        ax.spines[spine].set_linewidth(1.1)

    # Add legend
    ax.legend()
    

    plt.savefig(path)
    print(f"plot saved at {path}")



#imports
from .experiment_class import Experiment
from .load_config import load_config
from .make_plot_dir import make_plot_dir
from .make_data_dir import make_data_dir
from .plot_density import plot_density
from .plot_flower import plot_flower


__version__ = "0.1.7"
__all__ = [
    "Experiment",
    "load_config",
    "make_plot_dir",
    "make_data_dir",
    "plot_density",
    "plot_flower"
]


import os
import json
import importlib.resources as pkg_resources

def _load_config():
    # Check for the development config file first
    dev_path = os.path.join(os.path.dirname(__file__), 'config.json')
    
    if os.path.exists(dev_path):
        try:
            with open(dev_path, 'r') as f:
                config = json.load(f)
            return config['paths']  # Return all paths as a dictionary
        except (KeyError, json.JSONDecodeError):
            print("Error loading configuration. Using default paths.")
            return get_default_paths()  # Fallback to default paths
    else:
        # Fallback to package resource loading
        return load_config_from_package()

def load_config_from_package():
    try:
        with pkg_resources.open_text('rotational_placement', 'config.json') as f:
            config = json.load(f)
        return config['paths']  # Return all paths
    except (FileNotFoundError, KeyError, json.JSONDecodeError):
        print("Error loading configuration from package. Using default paths.")
        return get_default_paths()

def get_default_paths():
    return {
        "plot_save_path": "./plots",
        "data_files_save_path":"./data_files"
    }
